// DtL - Derive to LaTeX by Gary Bishop 29 June 1999

static char rcsid[] = "$Id: Str.cpp,v 1.1 1999-07-01 11:03:10-04 gb Exp gb $";

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "Str.h"

char* strsave(const char* s)
{
  int n = strlen(s);
  char* r = new char[n+1];
  strcpy(r, s);
  return r;
}

Str::Str(int n)
{
  p = new char[n+1];
  p[n] = 0;
  len = n;
}

Str:: Str()
{ 
  p = strsave("");
  len = 0;
}

Str::Str(const Str& s)
{
  p = strsave(s.p);
  len=s.len;
}

Str::Str(const char* s)
{
  p = strsave(s);
  len=strlen(s);
}

Str::Str(char c)
{
  p = new char[2];
  p[0] = c;
  p[1] = 0;
  len = 1;
}

Str::~Str()
{
  delete [] p;
  p=0;
  len=0;
}

Str Str::operator=(const Str& a)
{
  char *np = strsave(a.p);
  delete [] p;
  p = np;
  len=a.len;
  return *this;
}

Str Str::operator=(const char* a)
{
  delete [] p;
  p = strsave(a);
  len=strlen(a);
  return *this;
}

Str operator+(const Str& a, const Str& b)
{
  Str r(a.length() + b.length());
  strcpy(r.p, a.p);
  strcat(r.p, b.p);
  return r;
}
  
char& Str::operator[](int i)
{
  int l = length();
  if(i >= 0 && i < l) 
    return p[i];
  else if(i < 0 && i > -l)
    return p[l - i];
  else {
    fprintf(stderr, "Str::operator[] index out of bounds");
    exit(1);
  }
}

Str Str::operator+=(const Str& a)
{
  int nl = length() + a.length();
  char* np = new char[nl + 1];
  strcpy(np, p);
  strcat(np, a.p);
  delete [] p;
  p = np;
  len = nl;
  return *this;
}

Str operator*(const Str& a, int n)
{
  Str result;
  for(int i=0; i<n; i++)
    result += a;
  return result;
}

int isalllower(const Str& s)
{
  const char* p = s;
  while(*p && islower(*p)) p++;
  return *p == 0;
}

int isallupper(const Str& s)
{
  const char* p = s;
  while(*p && isupper(*p)) p++;
  return *p == 0;
}

int isallalnum(const Str& s)
{
  const char* p = s;
  while(*p && isalnum(*p)) p++;
  return *p == 0;
}

int isallascii(const Str& s)
{
  const char* p = s;
  while(*p && isascii(*p)) p++;
  return *p == 0;
}



