//  DtL - Derive to LaTeX by Gary Bishop 29 June 1999

/* parser for Derive internal form */

%{
static char rcsid[] = "$Id: parser.y,v 1.2 1999-07-02 16:15:24-04 gb Exp gb $";

#define YYDEBUG 1
#include <stdio.h>
#include "exp.h"
#include "DtL.h"

extern void yyerror(char*s);
extern int yylex();

extern output_latex(const Str& s);

%}
%union {
  char* s;
  exp* e;
  exp_list* l;
}
%token EOL
%token <s> VAR
%token <s> NUM
%token INT LIM SUM PROD DIF
%token <s> SYMBOL
%token <s> FUNC
%token ELEMENTOF
%token <s> COMMENT

%left OR
%left AND
%left LEQ GEQ '=' NEQ '<' '>'
%left '-' '+'
%left '*' '/'
%left NEG SQRT FLOOR CEILING ABS APOSTROPHE
%right '^'
%right SUB
%type <e> expr
%type <l> elist
%type <s> comment

%%
stmts   : stmt
	| stmts stmt
;

eols	: 
	| eols EOL
;

comment : /* empty */			{ $$ = ""; }
	| COMMENT			{ $$ = $1; }
;

stmt	: comment expr eols		{ DtL_output($1, LeftString + $2->latex() + RightString); }
	| comment VAR ELEMENTOF expr eols 	{ /* do nothing */ }
;

expr	: NUM				{ $$ = new exp_const($1); }
	| VAR				{ $$ = new exp_var($1); }
	| expr '=' expr			{ $$ = new exp_binary("=", 5, $1, $3); }
	| expr LEQ expr			{ $$ = new exp_binary("\\leq", 5, $1, $3); }
	| expr GEQ expr			{ $$ = new exp_binary("\\geq", 5, $1, $3); }
	| expr NEQ expr			{ $$ = new exp_binary("\\neq", 5, $1, $3); }
	| expr '<' expr			{ $$ = new exp_binary("<", 5, $1, $3); }
	| expr '>' expr			{ $$ = new exp_binary(">", 5, $1, $3); }
	| expr OR expr			{ $$ = new exp_binary("\\vee", 12, $1, $3); }
	| expr AND expr			{ $$ = new exp_binary("\\wedge", 14, $1, $3); }
	| expr '+' expr 		{ $$ = new exp_binary("+", 20, $1, $3); }
	| expr '-' expr 		{ $$ = new exp_binary("-", 20, $1, $3); }
	| expr '*' expr 		{ $$ = new exp_times($1, $3); }
	| expr '/' expr 		{ $$ = new exp_divide($1, $3); }
	| '-' expr %prec NEG		{ $$ = new exp_unary("-", $2); }
	| '+' expr %prec NEG		{ $$ = new exp_unary("+", $2); }
        | APOSTROPHE expr %prec NEG	{ $$ = new exp_unary("\\,'", $2); }
	| '(' expr ')' 			{ $$ = $2; }
	| VAR '(' elist ')' 		{ $$ = new exp_call($1, $3, 60); }
        | SQRT expr 		        { $$ = new exp_call("SQRT", $2, 0); }
	| FLOOR expr 			{ $$ = new exp_bracket("\\left\\lfloor",
							       "\\right\\rfloor", $2); }
	| FLOOR '(' expr ',' expr ')'	{ $$ = new exp_bracket("\\left\\lfloor",
							       "\\right\\rfloor",
							       new exp_divide($3,$5)); }
	| CEILING expr			{ $$ = new exp_bracket("\\left\\lceil",
							       "\\left\\rceil", $2); }
	| CEILING '(' expr ',' expr ')'	{ $$ = new exp_bracket("\\left\\lceil",
							       "\\right\\rceil",
							       new exp_divide($3,$5)); }
	| ABS expr			{ $$ = new exp_bracket("\\left|", "\\right|", $2); }
	| FUNC '(' expr ')' 		{ $$ = new exp_call($1, $3, 60); }
	| expr ':' '=' expr 		{ $$ = new exp_binary(":=", 0, $1, $4); }
	| '[' elist ']' 		{ $$ = new exp_vector($2); }
	| INT '(' elist ')' 		{ $$ = new exp_integral($3); }
	| SUM '(' elist ')'		{ $$ = new exp_sumprod("\\sum", $3); }
	| PROD '(' elist ')'		{ $$ = new exp_sumprod("\\prod", $3); }
	| DIF '(' expr ',' expr ')'	{ $$ = new exp_times(new exp_divide(new exp_var("d"),
						 new exp_times(new exp_var("d"), $5)), $3); }
	| LIM '(' elist ')'		{ $$ = new exp_limit($3); }
	| expr SUB expr 		{ $$ = new exp_script("_", $1, $3); }
	| expr '^' expr 		{ $$ = new exp_script("^", $1, $3); }
;

elist	: expr				{ $$ = new exp_list($1); }
	| expr ',' elist		{ $$ = new exp_list($1, $3); }
;
